import os
import sys
import requests
import argparse
from json import dumps

from .tasks_sh import Tasks_sh


version = VERSION = __version__ = '0.0.2'


def main():

    parent = argparse.ArgumentParser(add_help=False)
    parent.add_argument('--json',     action="store_true", default=False, help="return results as json")
    parent.add_argument('--slug',     default=os.getenv("TASKS_SH_SLUG", None), help="", type=str)
    parent.add_argument('--url',      default=os.getenv("TASKS_SH_URL", "https://tasks.sh"), help="change endpoint url", type=str)
    parent.add_argument('--auth',     default=os.getenv("TASKS_SH_AUTH", None), help="your authorization token", type=str)

    parser = argparse.ArgumentParser(prog='tasks.sh',
                                     parents=[parent],
                                     description="---\nDesciprtion\n---",
                                     formatter_class=argparse.RawDescriptionHelpFormatter,
                                     epilog="@tasks_sh - https://tasks.sh")

    subparsers = parser.add_subparsers(title='==== tasks.sh Commands')

    def subparse(endpoint, ignore=[], **more):
        new_parser = subparsers.add_parser(endpoint, parents=[parent], help="# here")
        new_parser.set_defaults(func=endpoint)
        #custom
        for arg, kwargs in more.items():
            if arg[0]=='_':
                new_parser.add_argument("--%s"%arg[1:], "-%s"%arg[1], **kwargs)
            else:
                new_parser.add_argument(arg, **kwargs)

        return new_parser
    
    # general
    # =======
    subparse('create') # add this repo or --slug
    subparse('login')
    # subparse('run')
    subparse('status')
    subparse('update')
    subparse('script:parse', script=dict(nargs="?"))
    subparse('script:validate', script=dict(nargs="?"))
    subparse('version')
    subparse('event', event=dict(nargs=1), 
                      _data=dict(nargs="?"),
                      _objects=dict(nargs="?"))
    subparse('help')
    subparse('logs', _tail=dict(action="store_true", default=False, help="continually stream logs"))

    # tokens
    # ======
    subparse('tokens')
    subparse('tokens:info', token=dict(nargs=1, help="", type=str))
    subparse('tokens:create', scope=dict(nargs="+", help="", type=str),
                              _ownerid=dict(nargs="?"))
    subparse('tokens:delete', token=dict(nargs="+", help="", type=str))

    # services
    # ========
    subparse('services')
    subparse('services:info', service=dict(nargs=1, help="", type=str))
    subparse('services:add', service=dict(nargs=1, help="add new services", type=str))
    subparse('services:update', service=dict(nargs=1, help="", type=str))

    # objects
    # ========
    subparse('objects')
    subparse('objects:info', object=dict(nargs=1, help="", type=str))
    subparse('objects:update', object=dict(nargs=1, help="", type=str))

    # releases
    # ========
    subparse('releases')
    subparse('releases:info', release=dict(nargs="?", help="", type=str))
    subparse('rollback', release=dict(nargs="?", help="rollback to a specific release number", type=str))
    subparse('deploy', commit=dict(nargs="?", help="deploy a commit sha", type=str))

    # builds
    # ======
    subparse('builds')
    subparse('builds:info', build=dict(nargs=1))

    # maintenance
    # ===========
    subparse('maintenance')
    subparse('maintenance:on')
    subparse('maintenance:off')

    # labs
    # ====
    # subparse('labs')
    # subparse('labs:info')
    # subparse('labs:enable')
    # subparse('labs:disable')

    if len([a for a in sys.argv if a[0]!='-']) > 1:
        args = parser.parse_args()
        if args.func == 'help':
            parser.print_help()
        else:
            # build token api object
            task = Tasks_sh(args)

            res = getattr(task, args.func.replace(':','_'))(**dict([(k,v) for k,v in args._get_kwargs() if k not in ("auth", "url", "slug", "func", "_branch", "_head", "provider", "json")]))
            if res is None:
                return

            elif isinstance(res, requests.Response):
                if args.json:
                    res = res.json()
                else:
                    res = res.text

            if type(res) in (list, dict):
                sys.stdout.write(dumps(res, indent=2, sort_keys=True))
            elif res is True:
                sys.stdout.write("true")
            elif res is False:
                sys.stdout.write("false")
                sys.exit(1)
            else:
                sys.stdout.write(res.strip()+"\n")
    else:
        parser.print_help()

    sys.stdout.write("\n") # for a pretty shell


if __name__ == '__main__':
    main()
