import os
import sys
import requests
import taskscript
from json import dumps
from json import loads
from urllib import urlencode
from contextlib import closing

try:
    import subprocess32 as subprocess
except ImportError:
    import subprocess


version = VERSION = __version__ = '0.0.1'



def interactive(chunk=None):
    """Not sure how to accomplish this yet.

    ```python
    from tornado import gen
    from tornado.web import authenticated
    from tornado.web import stream_request_body

    from app.handlers.base import BaseHandler


    @stream_request_body
    class LogHandler(BaseHandler):
        @authenticated
        @gen.coroutine
        def get(self, owner, repo):
            print "@get", owner, repo
            from time import sleep
            for x in range(20):
                self.write(".")
                if x ==10:
                    self.write('~')
                self.flush()
                sleep(1)

        def on_connection_close(self):
            print "@on_connection_close"

        def data_received(self, chunk):
            print "@data_received", chunk
            self.write('hello')
            self.flush()
            print 'wrote hello'
    ```

    """
    try: 
        if chunk:
            yield chunk
        while True:
            data = raw_input()
            yield data
    except KeyboardInterrupt:
        yield ""


def list_shrink(lst):
    if None in lst:
        return lst[lst.index(None)+1:]
    return lst


class Tasks_sh(object):
    """
    """
    def __init__(self, cli=False, **kwargs):
        self.args = {}
        self.args.update(kwargs)
        self.using_cli = cli is not False
        if cli:
            self.args = dict(auth=cli.auth,  url=cli.url,
                             slug=cli.slug,  func=None,
                             _branch=None,   _head=None,
                             provider='gh',  json=cli.json)

            if not cli.slug:
                try:
                    remote = subprocess.check_output('git remote -v | grep github.com', shell=True).split('\n')[0]
                    self.args['owner'], self.args['repo'] = tuple(remote.split('github.com',1)[1][1:].split(' ',1)[0][:-4].split('/', 1))
                except:
                    pass

            if self.args['slug']:
                self.args['owner'], self.args['repo'] = tuple(self.args['slug'].split('/', 1))
        
    @property
    def branch(self):
        if self._branch:
            return self._branch
        branch = subprocess.check_output('git rev-parse --abbrev-ref HEAD', shell=True).strip()
        self.args['_branch'] = None if branch == 'HEAD' else branch
        return self._branch

    @property
    def head(self):
        if self._head:
            return self._head
        self.args["_head"] = subprocess.check_output('git rev-parse HEAD', shell=True).strip()
        return self._head

    def __getattr__(self, index):
        return self.args[index]

    def _fetch(self, method, more, stream=False, interact=False, query={}, body=None):
        if interact:
            body = interactive(dumps(body) if body else None)
            stream = True
            query['stream'] = 'y'
        else:
            body = dumps(body)
        query = ("?%s"%urlencode(query)) if query else ""

        if stream and self.json:
            raise Exception("please remove --json flag for this endpoint")
        
        res = getattr(requests, method)("".join(("/".join((self.url,)+list_shrink((self.provider, self.owner, self.repo) + more)), query)),
                                        headers={"User-Agent": "tasks.sh v%s"%version, 
                                                 "Accept": "application/json" if self.json else "text/plain",
                                                 "Authorization": "token %s"%self.auth},
                                        data=body, stream=stream)
        return res

    def write(self, message):
        if self.using_cli and not self.json:
            sys.stdout.write(message)

    def loads(self, data):
        if type(data) in (list, dict):
            return data
        return loads(data)

    # Starting Endpoints
    # ==================
    def create(self, slug=None):
        return self._fetch('post', (None, self.provider, self.owner, self.repo))

    def login(self):
        sys.stdout.write("login feature not implemented atm")

    def script(self, script=None):
        if script:
            if script.startswith('@'):
                with open(os.path.join(os.getcwd(), script.replace('@','')), 'r') as r:
                    script = r.read()
        else:
            script = sys.stdin.read()

        script = script.replace('\\n','\n').replace('\\t','\t')
        return taskscript.parse(script)

    def script_parse(self, script=None):
        """validate taskscript file or text
        $ cat file.task | tasks.sh script:parse
        $ tasks.sh validate @file.task
        """
        return self.script(script).json()

    def script_validate(self, script=None):
        try:
            self.script(script)
            return True
        except:
            return False

    def status(self):
        return self._fetch('get', (None, 'status'))

    def event(self, event, data=None, objects=None):
        return self._fetch('post', (None, 'events'), 
                           body=dict(event=event[0], 
                                     objects=self.loads(objects) if objects else None, 
                                     data=self.loads(data) if data else None))

    def update(self):
        sys.stdout.write("plz run `pip install -U tasks.sh` thank you!")

    def version(self):
        return version

    def rollback(self, release=None):
        if not release:
            release = self._fetch('get', ('releases',), query=dict(limit=1)).json()['releases'][0]['number']
        return self._fetch('post', ('deploy',), body=dict(number=release))

    def run(self):
        sys.stdout.write("feature not implemented atm")

    def help(self):
        sys.stdout.write(Tasks_sh.__doc__)

    def deploy(self, commit):
        if not commit:
            commit = self.head
        if len(commit) < 40:
            raise ValueError("sha must be 40 characters long")
        self.write("Deploying %s/%s at %s...\n\n" % (self.owner, self.repo, commit))
        return self._fetch('post', ('deploy',), body=dict(commit=commit))

    def logs(self, tail=False):
        res = self._fetch('get', ('logs',), stream=tail)
        if tail:
            with closing(res) as r:
                # https://github.com/kennethreitz/requests/issues/844
                for chunk in r.iter_content(1):
                    if chunk:
                        sys.stdout.write(chunk)
        else:
            return res

    # services
    # --------
    def services(self):
        return self._fetch('get', ('services',))

    def services_info(self, service):
        return self._fetch('get', ('services', service[0]))

    def services_add(self, service):
        self.write("Preparing %s for %s/%s...\n" % (service[0], self.owner, self.repo))
        self.json = True
        res = self._fetch('options', ('services', service[0])).json()
        sys.stdout.write(res.get('message', '')+"\n")
        try:
            body = dict([(data['key'], raw_input(data['prompt'])) for data in res['config']])
        except KeyboardInterrupt:
            return
        self.json = False
        return self._fetch('post', ('services', service[0]), body=body)

    def services_update(self, service):
        pass

    # objects
    # -------
    def objects(self):
        return self._fetch('get', ('objects',))

    def objects_info(self, object):
        return self._fetch('get', ('objects', object[0]))

    def objects_update(self, object):
        pass

    # builds
    # ------
    def builds(self):
        return self._fetch('get', ('builds',))
    
    def builds_info(self, build):
        return self._fetch('get', ('builds', build[0]))

    # releases
    # --------
    def releases(self):
        return self._fetch('get', ('releases',))

    def releases_info(self, release):
        return self._fetch('get', ('releases', release[0]))

    # maintenance
    # -----------
    def maintenance(self):
        return self._fetch('get', ('maintenance',))

    def maintenance_on(self):
        self.write("Enabling maintenance mode for %s/%s... " % (self.owner, self.repo))
        return self._fetch('put', ('maintenance',), body=dict(toggle='on'))

    def maintenance_off(self):
        self.write("Disabling maintenance mode for %s/%s... " % (self.owner, self.repo))
        return self._fetch('put', ('maintenance',), body=dict(toggle='off'))

    # tokens
    # ------
    def tokens(self):
        return self._fetch('get', ('tokens', ))

    def tokens_info(self, token):
        return self._fetch('get', ('tokens', token[0]))

    def tokens_create(self, scope, ownerid=None):
        return self._fetch('post', ('tokens', ), body=dict(scope=scope, ownerid=ownerid))

    def tokens_delete(self, token):
        return self._fetch('delete', ('tokens', ",".join(token)))

    # labs
    # ----
    def labs(self):
        pass

    def labs_info(self):
        pass

    def labs_enable(self):
        pass

    def labs_disable(self):
        pass
